<?php

namespace Drupal\Tests\acquia_dam\FunctionalJavascript;

use Behat\Mink\Element\NodeElement;
use Drupal\acquia_dam\Entity\MediaSourceField;
use Drupal\Core\Template\Attribute;
use Drupal\media\Entity\Media;
use Drupal\Tests\ckeditor5\Traits\CKEditor5TestTrait;

final class CKEditor5IntegrationTest extends AcquiaDamWebDriverTestBase {

  use CKEditor5TestTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'classy';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'ckeditor5',
  ];

  /**
   * Tests the embed code dropdown in the editor for media elements.
   */
  public function testEmbedCodeDropdown(): void {
    $this->createAndLoginContentCreator();
    $this->grantSiteRegistrationToken();
    $this->grantCurrentUserDamToken();

    $media = Media::create([
      'bundle' => 'acquia_dam_pdf_asset',
      MediaSourceField::SOURCE_FIELD_NAME => [
        'asset_id' => '0324b0b2-5293-4aa0-b0aa-c85b003395e2',
      ],
    ]);
    $media->save();

    $this->drupalGet('/node/add/page');
    $this->getSession()->getPage()->fillField('Title', 'Embed test');
    $this->waitForEditor();
    $this->setEditorContent(
      $this->getDrupalMediaString($media->uuid(), 'link_text')
    );
    $this->assertEditorContent(
      $this->getDrupalMediaString($media->uuid(), 'link_text')
    );

    $drupalmedia = $this->assertSession()
      ->waitForElementVisible('css', '.ck-content .ck-widget.drupal-media');
    self::assertNotNull($drupalmedia);
    $drupalmedia->click();
    self::assertTrue($drupalmedia->hasClass('ck-widget_selected'));
    $this->assertVisibleBalloon('[aria-label="Drupal Media toolbar"]');
    $this->getBalloonButton('Text linked to viewer without download')->click();
    self::assertNotNull(
      $this->assertSession()->waitForElementVisible('css', '.ck-dropdown__panel-visible')
    );
    $this->getBalloonButton('Thumbnail linked to viewer without download')->click();
    $this->assertEditorContent(
      $this->getDrupalMediaString($media->uuid(), 'link_thumbnail')
    );
  }

  /**
   * Tests the "Update" button for revisions.
   */
  public function testUpdateMediaButton(): void {
    $this->grantSiteRegistrationToken();

    // Only images have version support, but the only fixture with multiple
    // versions is the video.
    $media = Media::create([
      'bundle' => 'acquia_dam_pdf_asset',
      MediaSourceField::SOURCE_FIELD_NAME => [
        'asset_id' => 'a56fb261-8ad5-4e0d-8323-0e8a3659ed39',
        'version_id' => 'ed84ab8e-a822-471d-b2f4-21b7353e4d0c',
        'external_id' => '8a1ouvfch',
      ],
    ]);
    $media->save();
    $first_revision = $media->getRevisionId();
    $media->setNewRevision();
    $media->set(MediaSourceField::SOURCE_FIELD_NAME, [
      'asset_id' => 'a56fb261-8ad5-4e0d-8323-0e8a3659ed39',
      'version_id' => 'e37312f5-e761-4ec4-92f2-8f0407f57bc7',
      'external_id' => '8a1ouvfch',
    ]);
    $media->save();
    $second_revision = $media->getRevisionId();
    self::assertNotEquals($first_revision, $second_revision);

    $this->createAndLoginContentCreator();
    $this->grantCurrentUserDamToken();

    $this->drupalGet('/node/add/page');
    $this->getSession()->getPage()->fillField('Title', 'Embed test');
    $this->waitForEditor();

    $this->setEditorContent(
      $this->getDrupalMediaString($media->uuid(), 'link_text')
    );
    $this->click('.ck-widget.drupal-media');
    $this->assertVisibleBalloon('[aria-label="Drupal Media toolbar"]');
    $this->assertBalloonButtonNotExists('Update media');

    $this->setEditorContent(
      $this->getDrupalMediaString($media->uuid(), 'link_text', $first_revision)
    );
    $this->assertEditorContent(
      $this->getDrupalMediaString($media->uuid(), 'link_text', $first_revision)
    );
    $this->click('.ck-widget.drupal-media');
    $this->assertVisibleBalloon('[aria-label="Drupal Media toolbar"]');
    $this->getBalloonButton('Update media');

    $this->setEditorContent(
      $this->getDrupalMediaString($media->uuid(), 'link_text', $second_revision)
    );
    $this->click('.ck-widget.drupal-media');
    $this->assertVisibleBalloon('[aria-label="Drupal Media toolbar"]');
    $this->assertBalloonButtonNotExists('Update media');

    $this->setEditorContent(
      $this->getDrupalMediaString($media->uuid(), 'link_text', $first_revision)
    );
    $this->click('.ck-widget.drupal-media');
    $this->assertVisibleBalloon('[aria-label="Drupal Media toolbar"]');
    $this->getBalloonButton('Update media')->click();
    $this->assertSession()->waitForElement('css', '#drupal-modal');
    $this->createScreenshot('../wtf.png');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertSession()->pageTextContains('You are about to update your current media to a newer version.');
    $this->assertSession()->pageTextContains('Current media');
    $this->assertSession()->pageTextContains('Updated media');
    $this->pressDialogButton('Update');
    $this->assertEditorContent(
      $this->getDrupalMediaString($media->uuid(), 'link_text', $second_revision)
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getBalloonButton(string $name): NodeElement {
    $button = $this->getSession()->getPage()
      ->find('css', '.ck-balloon-panel_visible .ck-balloon-rotator__content')
      // The parent method does not filter out hidden buttons which may have
      // the same name.
      ->find('xpath', "//button[not(contains(@class, 'ck-hidden'))][span[text()='$name']]");
    $this->assertNotEmpty($button);
    return $button;
  }

  /**
   * Asserts a particular balloon is not visible.
   *
   * @param string $name
   *   A CSS selector.
   */
  private function assertBalloonButtonNotExists(string $name): void {
    $button = $this->getSession()->getPage()
      ->find('css', '.ck-balloon-panel_visible .ck-balloon-rotator__content')
      // The parent method does not filter out hidden buttons which may have
      // the same name.
      ->find('xpath', "//button[not(contains(@class, 'ck-hidden'))][span[text()='$name']]");
    $this->assertEmpty($button);
  }

  /**
   * Asserts the CKEditor content directly.
   *
   * @param string $expected
   *   The expected content.
   */
  private function assertEditorContent(string $expected): void {
    self::assertSame(trim($expected), trim($this->getEditorDataAsHtmlString()));
  }

  /**
   * Sets the CKEditor content directly.
   *
   * @param string $content
   *   The content.
   */
  private function setEditorContent(string $content): void {
    $javascript = <<<JS
(function(){
  return Drupal.CKEditor5Instances.get(Drupal.CKEditor5Instances.keys().next().value).setData('$content');
})();
JS;
    $this->getSession()->evaluateScript($javascript);
  }

  /**
   * Gets the string for a <drupal-media> element.
   *
   * @param string $uuid
   *   The media entity UUID.
   * @param string $embed_code
   *   The embed code.
   * @param string $revision_id
   *   Optional: the revision ID for the media entity.
   *
   * @return string
   *   The <drupal-media> element string.
   */
  private function getDrupalMediaString(string $uuid, string $embed_code, string $revision_id = ''): string {
    // CKEditor sorts attributes alphabetically, so we also build our string
    // the same way to make testing easier.
    $attributes = new Attribute([
      'data-entity-type' => 'media',
      'data-entity-uuid' => $uuid,
      'data-entity-revision' => $revision_id,
      'data-embed-code-id' => $embed_code,
    ]);
    return "<drupal-media$attributes>&nbsp;</drupal-media>";
  }

}
